import copy
from typing import Optional, Tuple, List

import itertools
import numpy as np

_MARK_AS_CHAR = {0: '.', 1: 'x', 2: 'o'}


def opponent_mark(player: int):
    return -player + 3


class State(object):
    """
    Implementation of the game Five-in-a-row.
    """

    def __init__(self, board_size: int = 20):
        assert board_size >= 5
        self.board_size = board_size
        self.board_state = np.zeros((board_size, board_size))
        self.actions = list(range(board_size ** 2))
        self.int_mark = 1  # Track mark as an integer (1 or 2).
        self.history = []

    def current_player(self) -> int:
        return self.int_mark - 1

    def action_to_coord(self, a) -> Tuple[int, int]:
        return a // self.board_size, a % self.board_size

    def legal_actions(self) -> List[int]:
        return self.actions

    def apply_action(self, action):
        assert action in self.actions
        self.actions.remove(action)
        self.history.append(action)
        i, j = self.action_to_coord(action)
        self.board_state[i, j] = self.int_mark
        self.int_mark = opponent_mark(self.int_mark)

    def clone(self):
        return copy.deepcopy(self)

    def history_str(self) -> str:
        return str(self.history)

    def rewards(self) -> List[int]:
        player_int_mark = winner_int_mark(self.board_state)
        if player_int_mark is None:
            return [0, 0]
        else:
            sign = 1 if player_int_mark == 1 else -1
            return [1 * sign, -1 * sign]

    def is_terminal(self) -> bool:
        return not (self.board_state == 0).any() \
               or winner_int_mark(self.board_state) is not None

    def __str__(self) -> str:
        return self.repr(sep="\n")

    def serialize(self) -> str:
        return self.repr(sep="")

    def repr(self, sep) -> str:
        return sep.join("".join(_MARK_AS_CHAR[self.board_state[i, j]]
                                for j in range(self.board_size))
                        for i in range(self.board_size))


def eval_slices(slices) -> Optional[int]:
    for slice in slices:
        for key, group in itertools.groupby(slice):
            if key == 0: continue
            len_g = len(list(group))
            if len_g == 5: return key
    return None


def winner_int_mark(m: np.array) -> Optional[int]:
    """
    Find a winner if there is any based on the current board.
    Return the integer mark or None.
    """
    return eval_slices(
        # Make the evaluation lazy by chaining generator expressions.
        # Saves memory, as the lists of slices will not be explicitly allocated.
        itertools.chain(
            # Rows
            (m[row, :] for row in range(m.shape[0])),
            # Cols
            (m[:, col] for col in range(m.shape[1])),
            # Diag
            (m.diagonal(i) for i in range(m.shape[1] - 1, -m.shape[0], -1)),
            # Anti-diag
            (m[::-1, :].diagonal(i) for i in range(-m.shape[0] + 1, m.shape[1]))
        ))


class MCTSBot:
    def __init__(self, play_as: int):
        self.play_as = play_as

    def play_action(self, state: State):
        # TODO: implement MCTS bot according to the assignment task.
        return np.random.choice(state.legal_actions())


if __name__ == '__main__':
    board_size = int(input())
    play_as = int(input())
    state = State(board_size)
    bot = MCTSBot(play_as)
    while not state.is_terminal():
        if state.current_player() == play_as:
            action = bot.play_action(state)
            print(action)
        else:
            action = int(input())
        state.apply_action(action)
        # print(state) # For debugging.

    # Used for a final consistency check of the match (do not remove!)
    print("Terminal", state.serialize())
